Add-Type -AssemblyName "System.Net.Http"
Add-Type -AssemblyName "System.Text.Json"
Add-Type -AssemblyName "System.Security"

class Settings {
    static [string]$dummyRequest = "1"
    static [string]$merchantCcId = "{Set Merchant CcId here}"
    static [string]$merchantSecret = "{Set Merchant Secret Key here}"
    static [string]$targetHost = "https://api3.veritrans.co.jp"
    static [int]$timeoutSeconds = 30
    static [string]$userAgent = "VeriTrans4G/1.0.1 (PowerShell)"
    static [string]$txnVersion = "1.0.1"
}

$DebugPreference = "Continue"

class Main {

    static [void] ReadCsv([string]$command, [String]$path) {
        Write-Debug ([string]::Join($null, ((Get-Date), "[start processing]")))

        $dir = [System.IO.Path]::GetDirectoryName($path)
        $outputFileName = [System.IO.Path]::GetFileNameWithoutExtension($path) + "_" + (Get-Date).ToString("yyMMdd_HHmmss") + "_result.csv"
        $outputPath = Join-Path $dir $outputFileName
        $logFileName = "mdkps_" + (Get-Date).ToString("yyMMdd_HHmmss") + ".log"
        $logPath = Join-Path $dir $logFileName

        [System.Net.Http.HttpClient]$httpClient = [Helper]::CreateHttpClientInstance()
        [HttpSender]$httpSender = [HttpSender]::new($httpClient)
        [Transaction]$transaction = [Transaction]::new($httpSender, $logPath)

        $importCsv = Import-Csv -Path $path

        $resultList = @()

        [UInt32]$line = 0

        $importCsv | ForEach-Object {
            $line++
            Write-Debug ([string]::Join($null, ((Get-Date), "[csv line ${line}]", $_) ))

            $params = New-Object PSObject | Select-Object accountId, addCardInfo, amount, cardId, cardNumberMaskType, cleaningConfig, createDate, `
                defaultCard, defaultCardId, defaultCardOnly, deleteCardInfo, deleteDate, deleteOriginalAccountId, endDate, finalCharge, forceDeleteDate, groupId, jpo, oneTimeAmount, orderId, `
                originalAccountId, originalOrderId, recurringMemo1, recurringMemo2, recurringMemo3, startDate, token, updater, withAuthorize, withCapture

            if ($command -eq "account-add" -or $command -eq "account-update" -or $command -eq "account-restore") {
                $params.accountId = $_.accountId
                $params.createDate = [string]::IsNullOrWhiteSpace($_.createDate) -eq $true ? $null : $_.createDate

                [AbstractPayNowIdRequestDto]$dto
                if ($command -eq "account-add") {
                    $dto = [AccountAddRequestDto]::new()
                }
                if ($command -eq "account-update") {
                    $dto = [AccountUpdateRequestDto]::new()
                }
                if ($command -eq "account-restore") {
                    $dto = [AccountRestoreRequestDto]::new()
                }

                $dto.payNowIdParam = [PayNowIdParam]::new()
                $dto.payNowIdParam.accountParam = [AccountParam]::new()
                $dto.payNowIdParam.accountParam.accountId = $params.accountId

                if ($null -ne $params.createDate) {
                    $dto.payNowIdParam.accountParam.accountBasicParam = [AccountBasicParam]::new()
                    $dto.payNowIdParam.accountParam.accountBasicParam.createDate = $params.createDate
                }

                [PayNowResponse]$response = $transaction.SendCommand($dto)
                $resultList += [Helper]::PayNowResponseToPsObject($response, $params.accountId)
            }
            elseif ($command -eq "account-delete") {
                $params.accountId = $_.accountId
                $params.deleteDate = [string]::IsNullOrWhiteSpace($_.deleteDate) -eq $true ? $null : $_.deleteDate
                $params.forceDeleteDate = [string]::IsNullOrWhiteSpace($_.forceDeleteDate) -eq $true ? $null : $_.forceDeleteDate
                $params.deleteCardInfo = [string]::IsNullOrWhiteSpace($_.deleteCardInfo) -eq $true ? $null : $_.deleteCardInfo

                [AbstractPayNowIdRequestDto]$dto = [AccountDeleteRequestDto]::new()
                $dto.payNowIdParam = [PayNowIdParam]::new()
                $dto.payNowIdParam.accountParam = [AccountParam]::new()
                $dto.payNowIdParam.accountParam.accountId = $params.accountId

                if ($null -ne $params.deleteDate -or $null -ne $params.forceDeleteDate -or $null -ne $params.deleteCardInfo) {
                    $dto.payNowIdParam.accountParam.accountBasicParam = [AccountBasicParam]::new()
                }
                if ($null -ne $params.deleteDate) { $dto.payNowIdParam.accountParam.accountBasicParam.deleteDate = $params.deleteDate }
                if ($null -ne $params.forceDeleteDate) { $dto.payNowIdParam.accountParam.accountBasicParam.forceDeleteDate = $params.forceDeleteDate }
                if ($null -ne $params.deleteCardInfo) { $dto.payNowIdParam.accountParam.accountBasicParam.deleteCardInfo = $params.deleteCardInfo }

                [PayNowResponse]$response = $transaction.SendCommand($dto)
                $resultList += [Helper]::PayNowResponseToPsObject($response, $params.accountId)
            }
            elseif ($command -eq "account-link") {
                $params.accountId = $_.accountId
                $params.originalOrderId = $_.originalOrderId
                $params.createDate = [string]::IsNullOrWhiteSpace($_.createDate) -eq $true ? $null : $_.createDate
                $params.addCardInfo = [string]::IsNullOrWhiteSpace($_.addCardInfo) -eq $true ? $null : $_.addCardInfo
                $params.defaultCard = [string]::IsNullOrWhiteSpace($_.defaultCard) -eq $true ? $null : $_.defaultCard
                $params.updater = [string]::IsNullOrWhiteSpace($_.updater) -eq $true ? $null : $_.updater

                [AbstractPayNowIdRequestDto]$dto = [AccountLinkRequestDto]::new()
                $dto.payNowIdParam = [PayNowIdParam]::new()
                $dto.payNowIdParam.accountParam = [AccountParam]::new()
                $dto.payNowIdParam.accountParam.accountId = $params.accountId
                $dto.payNowIdParam.orderParam = [OrderParam]::new()
                $dto.payNowIdParam.orderParam.originalOrderId = $params.originalOrderId

                if ($null -ne $params.createDate -or $null -ne $params.addCardInfo) {
                    $dto.payNowIdParam.accountParam.accountBasicParam = [AccountBasicParam]::new()
                }
                if ($null -ne $params.createDate) { $dto.payNowIdParam.accountParam.accountBasicParam.createDate = $params.createDate }
                if ($null -ne $params.addCardInfo) { $dto.payNowIdParam.accountParam.accountBasicParam.addCardInfo = $params.addCardInfo }

                if ($null -ne $params.defaultCard -or $null -ne $params.updater) {
                    $dto.payNowIdParam.accountParam.cardParam = [CardParam]::new()
                }
                if ($null -ne $params.updater) { $dto.payNowIdParam.accountParam.cardParam.updater = $params.updater }
                if ($null -ne $params.defaultCard) { $dto.payNowIdParam.accountParam.cardParam.defaultCard = $params.defaultCard }

                [PayNowResponse]$response = $transaction.SendCommand($dto)
                $resultList += [Helper]::PayNowResponseToPsObject($response, $params.accountId)
            }
            elseif ($command -eq "account-copy") {
                $params.accountId = $_.accountId
                $params.originalAccountId = $_.originalAccountId
                $params.createDate = [string]::IsNullOrWhiteSpace($_.createDate) -eq $true ? $null : $_.createDate
                $params.deleteDate = [string]::IsNullOrWhiteSpace($_.deleteDate) -eq $true ? $null : $_.deleteDate
                $params.forceDeleteDate = [string]::IsNullOrWhiteSpace($_.forceDeleteDate) -eq $true ? $null : $_.forceDeleteDate
                $params.deleteCardInfo = [string]::IsNullOrWhiteSpace($_.deleteCardInfo) -eq $true ? $null : $_.deleteCardInfo
                $params.deleteOriginalAccountId = [string]::IsNullOrWhiteSpace($_.deleteOriginalAccountId) -eq $true ? $null : $_.deleteOriginalAccountId
                $params.addCardInfo = [string]::IsNullOrWhiteSpace($_.addCardInfo) -eq $true ? $null : $_.addCardInfo
                $params.defaultCardOnly = [string]::IsNullOrWhiteSpace($_.defaultCardOnly) -eq $true ? $null : $_.defaultCardOnly
                $params.cleaningConfig = [string]::IsNullOrWhiteSpace($_.cleaningConfig) -eq $true ? $null : $_.cleaningConfig

                [AbstractPayNowIdRequestDto]$dto = [AccountCopyRequestDto]::new()
                $dto.payNowIdParam = [PayNowIdParam]::new()
                $dto.payNowIdParam.accountParam = [AccountParam]::new()
                $dto.payNowIdParam.accountParam.accountId = $params.accountId
                $dto.payNowIdParam.accountParam.originalAccountId = $params.originalAccountId
            
                if ($null -ne $params.createDate -or $null -ne $params.deleteDate -or $null -ne $params.forceDeleteDate -or $null -ne $params.deleteCardInfo `
                        -or $null -ne $params.deleteOriginalAccountId -or $null -ne $params.addCardInfo -or $null -ne $params.defaultCardOnly `
                        -or $null -ne $params.cleaningConfig) {
                    $dto.payNowIdParam.accountParam.accountBasicParam = [AccountBasicParam]::new()
                }
                if ($null -ne $params.createDate) { $dto.payNowIdParam.accountParam.accountBasicParam.createDate = $params.createDate }
                if ($null -ne $params.deleteDate) { $dto.payNowIdParam.accountParam.accountBasicParam.deleteDate = $params.deleteDate }
                if ($null -ne $params.forceDeleteDate) { $dto.payNowIdParam.accountParam.accountBasicParam.forceDeleteDate = $params.forceDeleteDate }
                if ($null -ne $params.deleteCardInfo) { $dto.payNowIdParam.accountParam.accountBasicParam.deleteCardInfo = $params.deleteCardInfo }
                if ($null -ne $params.deleteOriginalAccountId) { $dto.payNowIdParam.accountParam.accountBasicParam.deleteOriginalAccountId = $params.deleteOriginalAccountId }
                if ($null -ne $params.addCardInfo) { $dto.payNowIdParam.accountParam.accountBasicParam.addCardInfo = $params.addCardInfo }
                if ($null -ne $params.defaultCardOnly) { $dto.payNowIdParam.accountParam.accountBasicParam.defaultCardOnly = $params.defaultCardOnly }
                if ($null -ne $params.cleaningConfig) { $dto.payNowIdParam.accountParam.accountBasicParam.cleaningConfig = $params.cleaningConfig }

                [PayNowResponse]$response = $transaction.SendCommand($dto)
                $resultList += [Helper]::PayNowResponseToPsObject($response, $params.accountId)
            }
            elseif ($command -eq "cardinfo-add") {
                $params.accountId = $_.accountId
                $params.defaultCard = [string]::IsNullOrWhiteSpace($_.defaultCard) -eq $true ? $null : $_.defaultCard
                $params.token = [string]::IsNullOrWhiteSpace($_.token) -eq $true ? $null : $_.token
                $params.withAuthorize = [string]::IsNullOrWhiteSpace($_.withAuthorize) -eq $true ? $null : $_.withAuthorize
                $params.updater = [string]::IsNullOrWhiteSpace($_.updater) -eq $true ? $null : $_.updater

                [AbstractPayNowIdRequestDto]$dto = [CardInfoAddRequestDto]::new()

                $dto.payNowIdParam = [PayNowIdParam]::new()
                $dto.payNowIdParam.accountParam = [AccountParam]::new()
                $dto.payNowIdParam.accountParam.accountId = $params.accountId

                if ($null -ne $params.defaultCard -or $null -ne $params.token -or $null -ne $params.withAuthorize -or $null -ne $params.updater) {
                    $dto.payNowIdParam.accountParam.cardParam = [CardParam]::new()
                    if ($null -ne $params.defaultCard) { $dto.payNowIdParam.accountParam.cardParam.defaultCard = $params.defaultCard }
                    if ($null -ne $params.token) { $dto.payNowIdParam.accountParam.cardParam.token = $params.token }
                    if ($null -ne $params.withAuthorize) { $dto.payNowIdParam.accountParam.cardParam.withAuthorize = $params.withAuthorize }
                    if ($null -ne $params.updater) { $dto.payNowIdParam.accountParam.cardParam.updater = $params.updater }
                }

                [PayNowResponse]$response = $transaction.SendCommand($dto)
                $resultList += [Helper]::PayNowResponseToCardInfoPsObject($response, $false, $params.accountId, $null)
            }
            elseif ($command -eq "cardinfo-get") {
                $params.accountId = $_.accountId
                $params.cardNumberMaskType = [string]::IsNullOrWhiteSpace($_.cardNumberMaskType) -eq $true ? $null : $_.cardNumberMaskType

                [AbstractPayNowIdRequestDto]$dto = [CardInfoGetRequestDto]::new()

                $dto.payNowIdParam = [PayNowIdParam]::new()
                $dto.payNowIdParam.accountParam = [AccountParam]::new()
                $dto.payNowIdParam.accountParam.accountId = $params.accountId

                if ($null -ne $params.cardNumberMaskType) {
                    $dto.payNowIdParam.accountParam.cardParam = [CardParam]::new()
                    $dto.payNowIdParam.accountParam.cardParam.cardNumberMaskType = $params.cardNumberMaskType
                }

                [PayNowResponse]$response = $transaction.SendCommand($dto)
                $resultList += [Helper]::PayNowResponseToCardInfoPsObject($response, $true, $params.accountId, $null)
            }
            elseif ($command -eq "cardinfo-update") {
                $params.accountId = $_.accountId
                $params.cardId = $_.cardId
                $params.defaultCard = [string]::IsNullOrWhiteSpace($_.defaultCard) -eq $true ? $null : $_.defaultCard
                $params.token = [string]::IsNullOrWhiteSpace($_.token) -eq $true ? $null : $_.token
                $params.withAuthorize = [string]::IsNullOrWhiteSpace($_.withAuthorize) -eq $true ? $null : $_.withAuthorize
                $params.updater = [string]::IsNullOrWhiteSpace($_.updater) -eq $true ? $null : $_.updater

                [AbstractPayNowIdRequestDto]$dto = [CardInfoUpdateRequestDto]::new()

                $dto.payNowIdParam = [PayNowIdParam]::new()
                $dto.payNowIdParam.accountParam = [AccountParam]::new()
                $dto.payNowIdParam.accountParam.accountId = $params.accountId
                $dto.payNowIdParam.accountParam.cardParam = [CardParam]::new()
                $dto.payNowIdParam.accountParam.cardParam.cardId = $params.cardId

                if ($null -ne $params.defaultCard) { $dto.payNowIdParam.accountParam.cardParam.defaultCard = $params.defaultCard }
                if ($null -ne $params.token) { $dto.payNowIdParam.accountParam.cardParam.token = $params.token }
                if ($null -ne $params.withAuthorize) { $dto.payNowIdParam.accountParam.cardParam.withAuthorize = $params.withAuthorize }
                if ($null -ne $params.updater) { $dto.payNowIdParam.accountParam.cardParam.updater = $params.updater }

                [PayNowResponse]$response = $transaction.SendCommand($dto)
                $resultList += [Helper]::PayNowResponseToCardInfoPsObject($response, $false, $params.accountId, $params.cardId)
            }
            elseif ($command -eq "cardinfo-delete") {
                $params.accountId = $_.accountId
                $params.cardId = $_.cardId
                $params.defaultCardId = [string]::IsNullOrWhiteSpace($_.defaultCardId) -eq $true ? $null : $_.defaultCardId

                [AbstractPayNowIdRequestDto]$dto = [CardInfoDeleteRequestDto]::new()

                $dto.payNowIdParam = [PayNowIdParam]::new()
                $dto.payNowIdParam.accountParam = [AccountParam]::new()
                $dto.payNowIdParam.accountParam.accountId = $params.accountId
                $dto.payNowIdParam.accountParam.cardParam = [CardParam]::new()
                $dto.payNowIdParam.accountParam.cardParam.cardId = $params.cardId

                if ($null -ne $params.defaultCardId) { $dto.payNowIdParam.accountParam.cardParam.defaultCardId = $params.defaultCardId }

                [PayNowResponse]$response = $transaction.SendCommand($dto)
                $resultList += [Helper]::PayNowResponseToCardInfoPsObject($response, $false, $params.accountId, $params.cardId)
            }
            if ($command -eq "recurring-add" -or $command -eq "recurring-update") {
                $params.accountId = $_.accountId
                $params.groupId = $_.groupId
                $params.cardId = [string]::IsNullOrWhiteSpace($_.cardId) -eq $true ? $null : $_.cardId
                $params.defaultCard = [string]::IsNullOrWhiteSpace($_.defaultCard) -eq $true ? $null : $_.defaultCard
                $params.updater = [string]::IsNullOrWhiteSpace($_.updater) -eq $true ? $null : $_.updater
                $params.startDate = [string]::IsNullOrWhiteSpace($_.startDate) -eq $true ? $null : $_.startDate
                $params.endDate = [string]::IsNullOrWhiteSpace($_.endDate) -eq $true ? $null : $_.endDate
                $params.oneTimeAmount = [string]::IsNullOrWhiteSpace($_.oneTimeAmount) -eq $true ? $null : $_.oneTimeAmount
                $params.amount = [string]::IsNullOrWhiteSpace($_.amount) -eq $true ? $null : $_.amount
                $params.recurringMemo1 = [string]::IsNullOrWhiteSpace($_.recurringMemo1) -eq $true ? $null : $_.recurringMemo1
                $params.recurringMemo2 = [string]::IsNullOrWhiteSpace($_.recurringMemo2) -eq $true ? $null : $_.recurringMemo2
                $params.recurringMemo3 = [string]::IsNullOrWhiteSpace($_.recurringMemo3) -eq $true ? $null : $_.recurringMemo3

                [AbstractPayNowIdRequestDto]$dto
                if ($command -eq "recurring-add") {
                    $dto = [RecurringAddRequestDto]::new()
                }
                if ($command -eq "recurring-update") {
                    $dto = [RecurringUpdateRequestDto]::new()
                }

                $dto.payNowIdParam = [PayNowIdParam]::new()
                $dto.payNowIdParam.accountParam = [AccountParam]::new()
                $dto.payNowIdParam.accountParam.accountId = $params.accountId
                $dto.payNowIdParam.accountParam.recurringChargeParam = [RecurringChargeParam]::new()

                $dto.payNowIdParam.accountParam.recurringChargeParam.groupId = $params.groupId

                if ($null -ne $params.startDate) { $dto.payNowIdParam.accountParam.recurringChargeParam.startDate = $params.startDate }
                if ($null -ne $params.endDate) { $dto.payNowIdParam.accountParam.recurringChargeParam.endDate = $params.endDate }
                if ($null -ne $params.oneTimeAmount) { $dto.payNowIdParam.accountParam.recurringChargeParam.oneTimeAmount = $params.oneTimeAmount }
                if ($null -ne $params.amount) { $dto.payNowIdParam.accountParam.recurringChargeParam.amount = $params.amount }
                if ($null -ne $params.recurringMemo1) { $dto.payNowIdParam.accountParam.recurringChargeParam.recurringMemo1 = $params.recurringMemo1 }
                if ($null -ne $params.recurringMemo2) { $dto.payNowIdParam.accountParam.recurringChargeParam.recurringMemo2 = $params.recurringMemo2 }
                if ($null -ne $params.recurringMemo3) { $dto.payNowIdParam.accountParam.recurringChargeParam.recurringMemo3 = $params.recurringMemo3 }

                if ($null -ne $params.cardId -or $null -ne $params.defaultCard -or $null -ne $params.updater) {
                    $dto.payNowIdParam.accountParam.cardParam ??= [CardParam]::new()
                    if ($null -ne $params.cardId) { $dto.payNowIdParam.accountParam.cardParam.cardId = $params.cardId }
                    if ($null -ne $params.defaultCard) { $dto.payNowIdParam.accountParam.cardParam.defaultCard = $params.defaultCard }
                    if ($null -ne $params.updater) { $dto.payNowIdParam.accountParam.cardParam.updater = $params.updater }
                }

                [PayNowResponse]$response = $transaction.SendCommand($dto)
                $resultList += [Helper]::RecurringPayNowResponseToPsObject($response, $params.accountId, $params.groupId)
            }
            if ($command -eq "recurring-get") {
                $params.accountId = $_.accountId

                $dto = [RecurringDeleteRequestDto]::new()

                $dto.payNowIdParam = [PayNowIdParam]::new()
                $dto.payNowIdParam.accountParam = [AccountParam]::new()
                $dto.payNowIdParam.accountParam.accountId = $params.accountId

                [PayNowResponse]$response = $transaction.SendCommand($dto)
                $resultList += [Helper]::RecurringGetPayNowResponseToPsObject($response, $params.accountId)
            }
            if ($command -eq "recurring-delete") {
                $params.accountId = $_.accountId
                $params.groupId = $_.groupId

                $params.endDate = [string]::IsNullOrWhiteSpace($_.endDate) -eq $true ? $null : $_.endDate
                $params.finalCharge = [string]::IsNullOrWhiteSpace($_.finalCharge) -eq $true ? $null : $_.finalCharge

                $dto = [RecurringDeleteRequestDto]::new()

                $dto.payNowIdParam = [PayNowIdParam]::new()
                $dto.payNowIdParam.accountParam = [AccountParam]::new()
                $dto.payNowIdParam.accountParam.accountId = $params.accountId
                $dto.payNowIdParam.accountParam.recurringChargeParam = [RecurringChargeParam]::new()

                $dto.payNowIdParam.accountParam.recurringChargeParam.groupId = $params.groupId

                if ($null -ne $params.endDate) { $dto.payNowIdParam.accountParam.recurringChargeParam.endDate = $params.endDate }
                if ($null -ne $params.finalCharge) { $dto.payNowIdParam.accountParam.recurringChargeParam.finalCharge = $params.finalCharge }

                [PayNowResponse]$response = $transaction.SendCommand($dto)
                $resultList += [Helper]::RecurringPayNowResponseToPsObject($response, $params.accountId, $params.groupId)
            }
            elseif ($command -eq "card-authorize") {
                $params.orderId = $_.orderId
                $params.amount = $_.amount
                $params.accountId = [string]::IsNullOrWhiteSpace($_.accountId) -eq $true ? $null : $_.accountId
                $params.cardId = [string]::IsNullOrWhiteSpace($_.cardId) -eq $true ? $null : $_.cardId
                $params.jpo = [string]::IsNullOrWhiteSpace($_.jpo) -eq $true ? $null : $_.jpo
                $params.withCapture = [string]::IsNullOrWhiteSpace($_.withCapture) -eq $true ? $null : $_.withCapture
                $params.token = [string]::IsNullOrWhiteSpace($_.token) -eq $true ? $null : $_.token

                $dto = [CardAuthorizeRequestDto]::new()
                $dto.orderId = $params.orderId
                $dto.amount = $params.amount
                if ($null -ne $params.jpo) { $dto.jpo = $params.jpo }
                if ($null -ne $params.withCapture) { $dto.withCapture = $params.withCapture }

                if ($null -ne $params.accountId) {
                    $dto.payNowIdParam ??= [PayNowIdParam]::new()
                    $dto.payNowIdParam.accountParam ??= [AccountParam]::new()
                    $dto.payNowIdParam.accountParam.accountId = $params.accountId    
                }

                if ($null -ne $params.cardId) {
                    $dto.payNowIdParam ??= [PayNowIdParam]::new()
                    $dto.payNowIdParam.accountParam ??= [AccountParam]::new()
                    $dto.payNowIdParam.accountParam.cardParam ??= [CardParam]::new()
                    $dto.payNowIdParam.accountParam.cardParam.cardId = $params.cardId
                }

                if ($null -ne $params.token) {
                    $dto.payNowIdParam ??= [PayNowIdParam]::new()
                    $dto.payNowIdParam.token = $params.token
                }

                [CardResponse]$response = $transaction.SendCommand($dto)
                $resultList += [Helper]::CardPayNowResponseToPsObject($response, $params.orderId)
            }
            elseif ($command -eq "card-reauthorize") {
                $params.orderId = $_.orderId
                $params.originalOrderId = $_.originalOrderId
                $params.amount = $_.amount
                $params.jpo = [string]::IsNullOrWhiteSpace($_.jpo) -eq $true ? $null : $_.jpo
                $params.withCapture = [string]::IsNullOrWhiteSpace($_.withCapture) -eq $true ? $null : $_.withCapture

                $dto = [CardReAuthorizeRequestDto]::new()
                $dto.orderId = $params.orderId
                $dto.originalOrderId = $params.originalOrderId
                $dto.amount = $params.amount
                if ($null -ne $params.jpo) { $dto.jpo = $params.jpo }
                if ($null -ne $params.withCapture) { $dto.withCapture = $params.withCapture }

                [CardResponse]$response = $transaction.SendCommand($dto)
                $resultList += [Helper]::CardResponseToPsObject($response, $params.orderId)
            }
            elseif ($command -eq "card-capture") {
                $params.orderId = $_.orderId
                $params.amount = $_.amount

                $dto = [CardCaptureRequestDto]::new()
                $dto.orderId = $params.orderId
                $dto.amount = $params.amount
                [CardResponse]$response = $transaction.SendCommand($dto)
                $resultList += [Helper]::CardResponseToPsObject($response, $params.orderId)
            }
            elseif ($command -eq "card-cancel") {
                $params.orderId = $_.orderId
                $params.amount = [string]::IsNullOrWhiteSpace($_.amount) -eq $true ? $null : $_.amount

                $dto = [CardCancelRequestDto]::new()
                $dto.orderId = $params.orderId
                if ($null -ne $params.amount) { $dto.amount = $params.amount }
                [CardResponse]$response = $transaction.SendCommand($dto)
                $resultList += [Helper]::CardResponseToPsObject($response, $params.orderId)
            }
        }

        $dir = [System.IO.Path]::GetDirectoryName($path)
        $outputFileName = [System.IO.Path]::GetFileNameWithoutExtension($path) + "_" + (Get-Date).ToString("yyMMdd_HHmmss") + "_result.csv"
        $outputPath = Join-Path $dir $outputFileName

        $resultList | Export-Csv $outputPath -NoTypeInformation 

        Write-Debug ([string]::Join($null, ((Get-Date), "[end processing]")))

        return
    }

}

#region mdk logic

class Transaction {

    [HttpSender]$HttpSender
    [string]$PayNowIdContext
    [string]$PayNowContext
    [string]$LogFilePath

    Transaction(
        [HttpSender]$httpSender,
        [string]$logFilePath
    ) {
        $this.HttpSender = $httpSender
        $this.LogFilePath = $logFilePath
        if ([Settings]::dummyRequest -eq "1") {
            $this.PayNowContext = "/test-paynow"
            $this.PayNowIdContext = "/test-paynowid"
        }
        else {
            $this.PayNowContext = "/paynow"
            $this.PayNowIdContext = "/paynowid"
        }
    }

    [CardResponse] SendCommand([AbstractPaymentCreditRequestDto]$dto) {
        try {
            $Error.Clear()
            $json = $this.Execute($dto)
            [CardResponse]$responseDto = [System.Text.Json.JsonSerializer]::Deserialize($json, [CardResponse], $this.SerializerOptions())
            return $responseDto
        }
        catch [MdkLocalException] {
            [MdkLocalException]$ex = $_.Exception
            $responseDto = [CardResponse]::new()
            $responseDto.result = [CardResponseResult]::new()
            $responseDto.result.vResultCode = $ex.ErrorCode + "000000000000"
            $responseDto.result.merrMsg = $ex.Message
            $responseDto.result.mstatus = "failure"
            $this.WriteLog($ex.Message)
            return $responseDto
        }
        catch [Exception] {
            [Exception]$ex = $_.Exception
            $responseDto = [CardResponse]::new()
            $responseDto.result = [CardResponseResult]::new()
            $responseDto.result.vResultCode = "MA99000000000000"
            $responseDto.result.merrMsg = $ex.Message
            $responseDto.result.mstatus = "failure"
            if ($ex -is [System.AggregateException]) {
                foreach ($innerException in $ex.Flatten().InnerException) {
                    $this.WriteLog($innerException.Message)
                    $this.WriteLog($innerException.StackTrace)
                }
            }
            else {
                $this.WriteLog($_.ex.Message)
                $this.WriteLog($_.ex.StackTrace)
            }
            return $responseDto
        }
    }

    [PayNowResponse] SendCommand([AbstractPayNowIdRequestDto]$dto) {
        try {
            $Error.Clear()
            $json = $this.Execute($dto)
            [PayNowResponse]$responseDto = [System.Text.Json.JsonSerializer]::Deserialize($json, [PayNowResponse], $this.SerializerOptions())
            return $responseDto
        }
        catch [MdkLocalException] {
            [MdkLocalException]$ex = $_.Exception
            $responseDto = [PayNowResponse]::new()
            $responseDto.result = [PayNowResponseResult]::new()
            $responseDto.result.vResultCode = $ex.ErrorCode + "000000000000"
            $responseDto.result.merrMsg = $ex.Message
            $responseDto.result.mstatus = "failure"
            $this.WriteLog($ex.Message)
            return $responseDto
        }
        catch [Exception] {
            [Exception]$ex = $_.Exception
            $responseDto = [PayNowResponse]::new()
            $responseDto.result = [PayNowResponseResult]::new()
            $responseDto.result.vResultCode = "MA99000000000000"
            $responseDto.result.merrMsg = $ex.Message
            $responseDto.result.mstatus = "failure"
            if ($ex -is [System.AggregateException]) {
                foreach ($innerException in $ex.Flatten().InnerException) {
                    $this.WriteLog($innerException.Message)
                    $this.WriteLog($innerException.StackTrace)
                }
            }
            else {
                $this.WriteLog($_.ex.Message)
                $this.WriteLog($_.ex.StackTrace)
            }
            return $responseDto
        }
    }

    [string] Execute($dto) {

        $url = $this.CreateSendUrl($dto)
        $this.WriteLog($url)

        $json = [System.Text.Json.JsonSerializer]::Serialize($dto, $this.SerializerOptions())

        $nodes = [System.Text.Json.Nodes.JsonNode]::Parse($json)
        $nodes.AsObject().Add("txnVersion", [System.Text.Json.Nodes.JsonValue]::Create([Settings]::txnVersion))
        $nodes.AsObject().Add("dummyRequest", [System.Text.Json.Nodes.JsonValue]::Create([Settings]::dummyRequest))
        $nodes.AsObject().Add("merchantCcid", [System.Text.Json.Nodes.JsonValue]::Create([Settings]::merchantCcId))

        $json = [System.Text.Json.JsonSerializer]::Serialize($nodes, $this.SerializerOptions())
        $this.WriteLog($json)

        $authHash = $this.ComputeAuthHash([Settings]::merchantCcId + $json + [Settings]::merchantSecret)

        $json = $this.AppendAuthHash($json, $authHash)

        [HttpResponse]$response = $this.HttpSender.Execute($url, $json)
        $this.WriteLog($response.ResponseBody)

        if ($response.StatusCode -eq [System.Net.HttpStatusCode]::OK) {
            # 200
            return $response.ResponseBody
        }
        elseif ($response.StatusCode -eq [System.Net.HttpStatusCode]::InternalServerError) {
            # 500
            throw [MdkLocalException]::new("MF05", "500 Internal Server Error")
        }
        elseif ($response.StatusCode -eq [System.Net.HttpStatusCode]::BadGateway) {
            # 502
            throw [MdkLocalException]::new("MF06", "502 Bad Gateway")
        }
        elseif ($response.StatusCode -eq [System.Net.HttpStatusCode]::ServiceUnavailable) {
            # 503
            throw [MdkLocalException]::new("MF07", "503 Service Unavailable")
        }
        else {
            throw [MdkLocalException]::new("MF02", "Could not connect to the GW Server")
        }
    }


    [string] CreateSendUrl($dto) {
        if ($dto -is [AccountAddRequestDto]) {
            return [Settings]::targetHost + $this.PayNowIdContext + "/v1/Add/account"
        }
        if ($dto -is [AccountUpdateRequestDto]) {
            return [Settings]::targetHost + $this.PayNowIdContext + "/v1/Update/account"
        }
        if ($dto -is [AccountDeleteRequestDto]) {
            return [Settings]::targetHost + $this.PayNowIdContext + "/v1/Delete/account"
        }
        if ($dto -is [AccountLinkRequestDto]) {
            return [Settings]::targetHost + $this.PayNowIdContext + "/v1/Link/account"
        }
        if ($dto -is [AccountRestoreRequestDto]) {
            return [Settings]::targetHost + $this.PayNowIdContext + "/v1/Restore/account"
        }
        if ($dto -is [AccountCopyRequestDto]) {
            return [Settings]::targetHost + $this.PayNowIdContext + "/v1/Copy/account"
        }
        if ($dto -is [CardInfoAddRequestDto]) {
            return [Settings]::targetHost + $this.PayNowIdContext + "/v1/Add/cardinfo"
        }
        if ($dto -is [CardInfoGetRequestDto]) {
            return [Settings]::targetHost + $this.PayNowIdContext + "/v1/Get/cardinfo"
        }
        if ($dto -is [CardInfoUpdateRequestDto]) {
            return [Settings]::targetHost + $this.PayNowIdContext + "/v1/Update/cardinfo"
        }
        if ($dto -is [CardInfoDeleteRequestDto]) {
            return [Settings]::targetHost + $this.PayNowIdContext + "/v1/Delete/cardinfo"
        }
        if ($dto -is [RecurringGetRequestDto]) {
            return [Settings]::targetHost + $this.PayNowIdContext + "/v1/Get/recurring"
        }
        if ($dto -is [RecurringAddRequestDto]) {
            return [Settings]::targetHost + $this.PayNowIdContext + "/v1/Add/recurring"
        }
        if ($dto -is [RecurringUpdateRequestDto]) {
            return [Settings]::targetHost + $this.PayNowIdContext + "/v1/Update/recurring"
        }
        if ($dto -is [RecurringDeleteRequestDto]) {
            return [Settings]::targetHost + $this.PayNowIdContext + "/v1/Delete/recurring"
        }
        if ($dto -is [CardAuthorizeRequestDto]) {
            return [Settings]::targetHost + $this.PayNowContext + "/v2/Authorize/card"
        }
        if ($dto -is [CardReAuthorizeRequestDto]) {
            return [Settings]::targetHost + $this.PayNowContext + "/v2/ReAuthorize/card"
        }
        if ($dto -is [CardCaptureRequestDto]) {
            return [Settings]::targetHost + $this.PayNowContext + "/v2/Capture/card"
        }
        if ($dto -is [CardCancelRequestDto]) {
            return [Settings]::targetHost + $this.PayNowContext + "/v2/Cancel/card"
        }
        throw [MdkLocalException]::new("MA99", "Send url does not create")
    }

    [string] AppendAuthHash([string]$json, [string]$authHash) {
        return "{`"params`":" + $json + ",`"authHash`":`"" + $authHash + "`"}"
    }

    [string] ComputeAuthHash([string] $data) {
        if ([string]::IsNullOrEmpty($data) -eq $true) {
            return ""
        }
        [System.Security.Cryptography.SHA256] $algorithm = $null
        try {
            $algorithm = [System.Security.Cryptography.SHA256]::Create()
            [byte[]]$result = $algorithm.ComputeHash([System.Text.Encoding]::UTF8.GetBytes($data))
            return [System.BitConverter]::ToString($result).ToLower().Replace("-", "")
        }
        finally {
            $algorithm.Dispose()
        }
    }

    [System.Text.Json.JsonSerializerOptions] SerializerOptions() {
        $options = [System.Text.Json.JsonSerializerOptions]::new()
        $options.DefaultIgnoreCondition = [System.Text.Json.Serialization.JsonIgnoreCondition]::WhenWritingNull
        return $options
    }

    [void] WriteLog([string]$message) {
        Write-Output ((Get-Date).ToString("O") + "`t" + $message) | Out-File -FilePath $this.LogFilePath -Append
    }

}

class HttpSender {

    [System.Net.Http.HttpClient]$Client

    HttpSender(
        [System.Net.Http.HttpClient]$client
    ) {
        $this.Client = $client
    }

    [HttpResponse] Execute([string]$url, [string]$requestBody) {

        $httpContent = New-Object -TypeName System.Net.Http.StringContent($requestBody, [System.Text.Encoding]::UTF8, "application/x-www-form-urlencoded")

        $task = $null
        try {
            #$Error.Clear()
            $task = $this.Client.PostAsync($url, $httpContent)
            $task.Wait()
            $response = $task.Result
            return [HttpResponse]::new($response.Content.ReadAsStringAsync().Result, $response.StatusCode)
        }
        catch [Exception] {
            throw
        }
        finally {
            $httpContent.Dispose()
        }

    }
}

class HttpResponse {
    HttpResponse([string]$responseBody, [System.Net.HttpStatusCode]$statusCode) {
        $this.ResponseBody = $responseBody
        $this.StatusCode = $statusCode
    }
    [string]$ResponseBody
    [System.Net.HttpStatusCode]$StatusCode
}

class MdkLocalException : Exception {

    [string]$ErrorCode
    [string]$Message

    MdkLocalException([string]$errorCode, [string]$message) {
        $this.ErrorCode = $errorCode
        $this.Message = $message
    }

}

#endregion

class Helper {

    static [System.Net.Http.HttpClient] CreateHttpClientInstance() {
        [System.Net.Http.HttpClient]$client = [System.Net.Http.HttpClient]::new()
        $client.Timeout = [System.TimeSpan]::FromSeconds([Settings]::timeoutSeconds)
        $client.DefaultRequestHeaders.Add("User-Agent", [Settings]::userAgent)
        return $client
    }

    static [PSObject] PayNowResponseToPsObject([PayNowResponse]$response, [string]$accountId) {
        $result = New-Object PSObject | Select-Object accountId, serviceType, mstatus, vResultCode, merrMsg, processId, status, message
        $result.accountId = $accountId
        if ($null -ne $response -and $null -ne $response.result) {
            $result.serviceType = $response.result.serviceType
            $result.mstatus = $response.result.mstatus
            $result.vResultCode = $response.result.vResultCode
            $result.merrMsg = $response.result.merrMsg
        }
        if ($null -ne $response -and $null -ne $response.payNowIdResponse) {
            $result.processId = $response.payNowIdResponse.processId
            $result.status = $response.payNowIdResponse.status
            $result.message = $response.payNowIdResponse.message
        }
        #$result.serviceType = ${response}?.{result}?.serviceType
        #$result.mstatus = ${response}?.{result}?.mstatus
        #$result.vResultCode = ${response}?.{result}?.vResultCode
        #$result.merrMsg = ${response}?.{result}?.merrMsg
        #$result.processId = ${response}?.{payNowIdResponse}?.processId
        #$result.status = ${response}?.{payNowIdResponse}?.status
        #$result.message = ${response}?.{payNowIdResponse}?.message
        #$result.accountId = ${response}?.{payNowIdResponse}?.{account}?.accountId
        return $result
    }

    static [PSObject[]] PayNowResponseToCardInfoPsObject([PayNowResponse]$response, [bool]$multiCardLine, [string]$accountId, [string]$cardId) {
        [bool]$accountObjectExists = $false
        
        $result = New-Object PSObject | Select-Object accountId, cardId, cardNumber, cardExpire, defaultCard, `
            serviceType, mstatus, vResultCode, merrMsg, processId, status, message
        $result.accountId = $accountId
        if ($null -ne $response -and $null -ne $response.result) {
            $result.serviceType = $response.result.serviceType
            $result.mstatus = $response.result.mstatus
            $result.vResultCode = $response.result.vResultCode
            $result.merrMsg = $response.result.merrMsg
        }
        if ($null -ne $response -and $null -ne $response.payNowIdResponse) {
            $result.processId = $response.payNowIdResponse.processId
            $result.status = $response.payNowIdResponse.status
            $result.message = $response.payNowIdResponse.message
            if ($null -ne $response.payNowIdResponse.account) {
                $accountObjectExists = $true
            }
        }

        $resultList = @()

        if ($true -eq $accountObjectExists -and $null -ne $response.payNowIdResponse.account.cardInfo `
                -and $response.payNowIdResponse.account.cardInfo.Length -gt 0) {
            if ($multiCardLine -eq $true) {
                foreach ($cardInfo in $response.payNowIdResponse.account.cardInfo) {
                    $cardInfoResult = $result.PSObject.Copy()
                    $cardInfoResult.cardId = $cardInfo.cardId
                    $cardInfoResult.cardNumber = $cardInfo.cardNumber
                    $cardInfoResult.cardExpire = $cardInfo.cardExpire
                    $cardInfoResult.defaultCard = $cardInfo.defaultCard
                    $resultList += $cardInfoResult
                }
            }
            else {
                $result.cardId = $response.payNowIdResponse.account.cardInfo[0].cardId
                $result.cardNumber = $response.payNowIdResponse.account.cardInfo[0].cardNumber
                $result.cardExpire = $response.payNowIdResponse.account.cardInfo[0].cardExpire
                $result.defaultCard = $response.payNowIdResponse.account.cardInfo[0].defaultCard
                $resultList += $result
            }
        }
        else {
            $result.cardId = $cardId
            $resultList += $result
        }

        return $resultList
    }

    static [PSObject] RecurringPayNowResponseToPsObject([PayNowResponse]$response, [string]$accountId, [string]$groupId) {
        [bool]$cardInfoObjectExists = $false
        
        $result = New-Object PSObject | Select-Object accountId, groupId, cardId, `
            serviceType, mstatus, vResultCode, merrMsg, processId, status, message
        $result.accountId = $accountId
        $result.groupId = $groupId
        if ($null -ne $response -and $null -ne $response.result) {
            $result.serviceType = $response.result.serviceType
            $result.mstatus = $response.result.mstatus
            $result.vResultCode = $response.result.vResultCode
            $result.merrMsg = $response.result.merrMsg
        }
        if ($null -ne $response -and $null -ne $response.payNowIdResponse) {
            $result.processId = $response.payNowIdResponse.processId
            $result.status = $response.payNowIdResponse.status
            $result.message = $response.payNowIdResponse.message
            if ($null -ne $response.payNowIdResponse.account) {
                if ($null -ne $response.payNowIdResponse.account.cardInfo) {
                    $cardInfoObjectExists = $true
                }
            }
        }
      
        if ($true -eq $cardInfoObjectExists -and $response.payNowIdResponse.account.cardInfo.Length -gt 0) {
            $result.cardId = $response.payNowIdResponse.account.cardInfo[0].cardId
        }

        return $result
    }
    
    static [PSObject[]] RecurringGetPayNowResponseToPsObject([PayNowResponse]$response, [string]$accountId) {
        [bool]$accountObjectExists = $false
        [bool]$recurringChargeObjectExists = $false
        [bool]$cardInfoObjectExists = $false
        
        $result = New-Object PSObject | Select-Object accountId, groupId, cardId, `
            serviceType, mstatus, vResultCode, merrMsg, processId, status, message
        $result.accountId = $accountId
        
        if ($null -ne $response -and $null -ne $response.result) {
            $result.serviceType = $response.result.serviceType
            $result.mstatus = $response.result.mstatus
            $result.vResultCode = $response.result.vResultCode
            $result.merrMsg = $response.result.merrMsg
        }
        if ($null -ne $response -and $null -ne $response.payNowIdResponse) {
            $result.processId = $response.payNowIdResponse.processId
            $result.status = $response.payNowIdResponse.status
            $result.message = $response.payNowIdResponse.message
            if ($null -ne $response.payNowIdResponse.account) {
                $accountObjectExists = $true
                if ($null -ne $response.payNowIdResponse.account.recurringCharge) {
                    $recurringChargeObjectExists = $true
                }
                if ($null -ne $response.payNowIdResponse.account.cardInfo) {
                    $cardInfoObjectExists = $true
                }
            }
        }

        $resultList = @()

        if ($true -eq $accountObjectExists -and $true -eq $recurringChargeObjectExists `
                -and $response.payNowIdResponse.account.recurringCharge.Length -gt 0) {
            foreach ($recurringCharge in $response.payNowIdResponse.account.recurringCharge) {
                $recurringChargeResult = $result.PSObject.Copy()
                $recurringChargeResult.cardId = $recurringCharge.cardId
                $recurringChargeResult.groupId = $recurringCharge.groupId
                $resultList += $recurringChargeResult
            }
        }
        else {
            $resultList += $result
        }

        return $resultList
    }


    static [PSObject] CardPayNowResponseToPsObject([CardResponse]$response, [string]$orderId) {
        $result = New-Object PSObject | Select-Object orderId, serviceType, mstatus, vResultCode, merrMsg, marchTxn, custTxn, `
            pending, loopback, resAuthCode, resCenterErrorCode, acquirerCode, accountId, cardId, cardNumber, cardExpire, defaultCard, `
            processId, status, message
        $result.orderId = $orderId

        if ($null -ne $response -and $null -ne $response.result) {
            $result.serviceType = $response.result.serviceType
            $result.mstatus = $response.result.mstatus
            $result.vResultCode = $response.result.vResultCode
            $result.merrMsg = $response.result.merrMsg
            $result.marchTxn = $response.result.marchTxn
            $result.custTxn = $response.result.custTxn
            $result.pending = $response.result.pending
            $result.loopback = $response.result.loopback
            $result.resAuthCode = $response.result.resAuthCode
            $result.resCenterErrorCode = $response.result.resCenterErrorCode
            $result.acquirerCode = $response.result.acquirerCode
        }
        if ($null -ne $response -and $null -ne $response.payNowIdResponse) {
            $result.processId = $response.payNowIdResponse.processId
            $result.status = $response.payNowIdResponse.status
            $result.message = $response.payNowIdResponse.message
            if ($null -ne $response.payNowIdResponse.account) {
                $result.accountId = $response.payNowIdResponse.account.accountId
                if ($null -ne $response.payNowIdResponse.account.cardInfo `
                        -and $null -ne $response.payNowIdResponse.account.cardInfo.Length -gt 0) {
                    $result.cardId = $response.payNowIdResponse.account.cardInfo[0].cardId
                    $result.cardNumber = $response.payNowIdResponse.account.cardInfo[0].cardNumber
                    $result.cardExpire = $response.payNowIdResponse.account.cardInfo[0].cardExpire
                    $result.defaultCard = $response.payNowIdResponse.account.cardInfo[0].defaultCard
                }
            }
        }
        return $result
    }

    static [PSObject] CardResponseToPsObject([CardResponse]$response, [string]$orderId) {
        $result = New-Object PSObject | Select-Object orderId, serviceType, mstatus, vResultCode, merrMsg, marchTxn, custTxn, `
            pending, loopback, resAuthCode, resCenterErrorCode, acquirerCode
        $result.orderId = $orderId

        if ($null -ne $response -and $null -ne $response.result) {
            $result.serviceType = $response.result.serviceType
            $result.mstatus = $response.result.mstatus
            $result.vResultCode = $response.result.vResultCode
            $result.merrMsg = $response.result.merrMsg
            $result.marchTxn = $response.result.marchTxn
            $result.custTxn = $response.result.custTxn
            $result.pending = $response.result.pending
            $result.loopback = $response.result.loopback
            $result.resAuthCode = $response.result.resAuthCode
            $result.resCenterErrorCode = $response.result.resCenterErrorCode
            $result.acquirerCode = $response.result.acquirerCode
        }
        return $result
    }

}


#region request dto class

class AbstractPayNowIdRequestDto {
    [PayNowIdParam]$payNowIdParam
}

# /test-paynowid/v1/Add/account
class AccountAddRequestDto : AbstractPayNowIdRequestDto {
}

# /test-paynowid/v1/Update/account
class AccountUpdateRequestDto : AbstractPayNowIdRequestDto {
}

# /test-paynowid/v1/Delete/account
class AccountDeleteRequestDto : AbstractPayNowIdRequestDto {
}

# /test-paynowid/v1/Link/account
class AccountLinkRequestDto : AbstractPayNowIdRequestDto {
}

# /test-paynowid/v1/Restore/account
class AccountRestoreRequestDto : AbstractPayNowIdRequestDto {
}

# /test-paynowid/v1/Copy/account
class AccountCopyRequestDto : AbstractPayNowIdRequestDto {
}

# /test-paynowid/v1/Add/cardinfo
class CardInfoAddRequestDto : AbstractPayNowIdRequestDto {
}

# /test-paynowid/v1/Get/cardinfo
class CardInfoGetRequestDto : AbstractPayNowIdRequestDto {
}

# /test-paynowid/v1/Update/cardinfo
class CardInfoUpdateRequestDto : AbstractPayNowIdRequestDto {
}

# /test-paynowid/v1/Delete/cardinfo
class CardInfoDeleteRequestDto : AbstractPayNowIdRequestDto {
}

# /test-paynowid/v1/Get/recurring
class RecurringGetRequestDto : AbstractPayNowIdRequestDto {
}

# /test-paynowid/v1/Add/recurring
class RecurringAddRequestDto : AbstractPayNowIdRequestDto {
}

# /test-paynowid/v1/Update/recurring
class RecurringUpdateRequestDto : AbstractPayNowIdRequestDto {
}

# /test-paynowid/v1/Delete/recurring
class RecurringDeleteRequestDto : AbstractPayNowIdRequestDto {
}


class AbstractPaymentRequestDto {
    [PayNowIdParam]$payNowIdParam
}

class AbstractPaymentCreditRequestDto : AbstractPaymentRequestDto {
}

# /test-paynow/v2/Authorize/card
class CardAuthorizeRequestDto : AbstractPaymentCreditRequestDto {
    [string]$orderId
    [string]$amount
    [string]$acquirerCode
    [string]$jpo
    [string]$withCapture
    [string]$salesDay
    [string]$itemCode
    [string]$currencyUnit
    [string]$firstPayment
    [string]$bonusFirstPayment
    [string]$mcAmount
}

# /test-paynow/v2/ReAuthorize/card
class CardReAuthorizeRequestDto : AbstractPaymentCreditRequestDto {
    [string]$orderId
    [string]$originalOrderId
    [string]$amount
    [string]$acquirerCode
    [string]$jpo
    [string]$withCapture
    [string]$salesDay
    [string]$itemCode
    [string]$currencyUnit
    [string]$firstPayment
    [string]$bonusFirstPayment
    [string]$mcAmount
}

# /test-paynow/v2/Capture/card
class CardCaptureRequestDto : AbstractPaymentCreditRequestDto {
    [string]$orderId
    [string]$amount
}

# /test-paynow/v2/Cancel/card
class CardCancelRequestDto : AbstractPaymentCreditRequestDto {
    [string]$orderId
    [string]$amount
}

class PayNowIdParam {
    [AccountParam]$accountParam
    [ChargeParam]$chargeParam
    [OrderParam]$orderParam
    [string]$tanking
    [string]$freeKey
    [string]$memo1
    [string]$receiptData
    [string]$token
}

class AccountParam {
    [string]$accountId
    [string]$onetimeToken
    [string]$onetimeTokenType
    [string]$originalAccountId
    [string]$payNowId
    [string]$transData
    [AccountBasicParam]$accountBasicParam
    [CardParam]$cardParam
    [RecurringChargeParam]$recurringChargeParam
}

class AccountBasicParam {
    [string]$createDate
    [string]$deleteDate
    [string]$forceDeleteDate
    [string]$deleteCardInfo
    [string]$addCardInfo
    [string]$deleteOriginalAccountId
    [string]$defaultCardOnly
    [string]$cleaningConfig
}

class CardParam {
    [string]$cardId
    [string]$defaultCard
    [string]$defaultCardId
    [string]$updater
    [string]$token
    [string]$cardNumberMaskType
    [string]$withAuthorize
}

class ChargeParam {
    [string]$groupId
    [string]$groupName
    [string]$type
    [string]$oneTimeAmount
    [string]$amount
    [string]$chargeType
    [string]$schedule
    [string]$salesDay
    [string]$acquireCode
}

class RecurringChargeParam {
    [string]$groupId
    [string]$startDate
    [string]$endDate
    [string]$finalCharge
    [string]$oneTimeAmount
    [string]$amount
    [string]$recurringMemo1
    [string]$recurringMemo2
    [string]$recurringMemo3
    [string]$useChargeOption
    [string]$salesDay
    [string]$acquireCode
}

class OrderParam {
    [string]$originalOrderId
    [string]$cleaningMerchantId
}

#endregion

#region response dto class

class CardResponse {
    [PayNowIdResponse]$payNowIdResponse
    [CardResponseResult]$result
}

class CardResponseResult {
    [string]$vResultCode
    [string]$custTxn
    [string]$acquirerCode
    [string]$cardTransactiontype
    [string]$centerRequestDate
    [string]$centerResponseDate
    [string]$connectedCenterId
    [string]$gatewayRequestDate
    [string]$gatewayResponseDate
    [string]$kindCode
    [string]$loopback
    [string]$pending
    [string]$reqAcquirerCode
    [string]$reqAmount
    [string]$reqCardExpire
    [string]$reqCardNumber
    [string]$reqItemCode
    [string]$reqJpoInformation
    [string]$reqWithCapture
    [string]$resActionCode
    [string]$resAuthCode
    [string]$resCenterErrorCode
    [string]$resReturnReferenceNumber
    [string]$marchTxn
    [string]$merrMsg
    [string]$mstatus
    [string]$orderId
    [string]$serviceType
    [string]$txnVersion
}


class PayNowResponse {
    [PayNowIdResponse]$payNowIdResponse
    [PayNowResponseResult]$result
}

class PayNowIdResponse {
    [string]$processId
    [string]$status
    [string]$message
    [Account]$account
}

class PayNowResponseResult {
    [string]$vResultCode
    [string]$merrMsg
    [string]$mstatus
    [string]$serviceType
    [string]$txnVersion
}


class Account {
    [string]$accountId
    [string]$onetimeToken
    [string]$onetimeTokenExpire
    [string]$transData
    [AccountBasic]$accountBasic
    [AccountInfo[]]$accountInfo
    [CardInfo[]]$cardInfo
    [RecurringCharge[]]$recurringCharge
}

class CardInfo {
    [string]$cardId
    [string]$cardNumber
    [string]$cardExpire
    [string]$cardholderName
    [string]$defaultCard
    [string]$acquireCode3
    [string]$kindCode
    [string]$originalCardId
}

class AccountInfo {
    [string]$accountInfoId
    [string]$accountType
    [string]$defaultAccount
    [string]$recipient
    [string]$zip
    [string]$address
    [string]$tel
}

class AccountBasic {
    [string]$lastName
    [string]$lastKanaName
    [string]$firstName
    [string]$firstKanaName
    [string]$mailAddress
    [string]$createDate
    [string]$deleteDate
}

class RecurringCharge {
    [string]$groupId
    [string]$cardId
}

#endregion

if ($Args.Count -eq 2) {

    $enableCommands = @('card-authorize', 'card-reauthorize', 'card-capture', 'card-cancel', 'account-add', 'account-update', `
            'account-delete', 'account-restore', 'account-link', 'account-copy', 'cardinfo-add', 'cardinfo-get', 'cardinfo-update', 'cardinfo-delete', `
            'recurring-get', 'recurring-add', 'recurring-update', 'recurring-delete')
    if ($enableCommands.Contains($Args[0]) -ne $true) {
        Write-Host "正しくないコマンドが指定されました。"
        exit 0
    }
    [Main]::ReadCsv($Args[0], $Args[1])
}
else {
    Write-Host "`n・利用方法`n"
    Write-Host "第1引数でコマンド種別、第2引数でファイル名を指定してください。"
    Write-Host "> .\mdk.ps1 card-authorize .\TestData.csv`n"
}

exit 0